local _, iTF = ...

--Defaults iTFConfig.frame.spacing = 4
-- iTFConfig.frame.height = 50
-- iTFConfig.frame.castBarHeight = 25
-- iTFConfig.statusbar.castBarHeight
function iTF:LoadDefaults(force)
	local defaults = {
		['anchor'] = {
			['x'] = 20,
			['y'] = 600,
		},
		['grow'] = 'TOPLEFT',
		['frame'] = {
			['width'] = 60,
			['height'] = 30,
			['spacing'] = 2,
			['castBarHeight'] = 8,
		},
		['raidIcon'] = {
			['size'] = 20,
		},
		['text'] = {
			['pos'] = 'TOPLEFT',
			['size'] = 11,
			['font'] = 'Fonts\\ARIALN.TTF',
		},
		['statusbar'] = {
			['texture'] = 'Interface\\Buttons\\WHITE8x8',
		},
		['colS'] = 5,
		['colors'] = {
			['backdrop'] = {
				['bg'] = {
					['r'] = 0.1,
					['g'] = 0.1,
					['b'] = 0.1,
					['a'] = 0.9,
				},
				['border'] = {
					['r'] = 0,
					['g'] = 0,
					['b'] = 0,
					['a'] = 1,
				},
			},
			['statusbar'] = {
				['main'] = {
					['r'] = 0.5,
					['g'] = 0,
					['b'] = 0,
					['a'] = 0.8,
				},
				['cast'] = {
					['r'] = 0.5,
					['g'] = 0.5,
					['b'] = 0.5,
					['a'] = 0.8,
				},
			},
			['text'] = {
				['main'] = {
					['r'] = 1,
					['g'] = 1,
					['b'] = 1,
					['a'] = 1,
				},
				['cast'] = {
					['r'] = 1,
					['g'] = 1,
					['b'] = 0,
					['a'] = 1,
				},
				['immune'] = {
					['r'] = 1,
					['g'] = 0,
					['b'] = 0,
					['a'] = 1,
				},
			},
			['targeting'] = {
				['main'] = {
					['r'] = 1,
					['g'] = 1,
					['b'] = 1,
					['a'] = 1,
				},
			},
		},
		['glows'] = {
			['outOfCombat'] = {
				['pos'] = 'glowLeft',
				['color'] = {0.5,0.5,0.5,0.8},
				['enable'] = true,
			},
		},
	}
	if force then
		iTFConfig = defaults
	else
		for k,v in pairs(defaults) do
			if iTFConfig[k] == nil then
				iTFConfig[k] = v
			elseif type(v) == 'table' then
				for deeperKey, deeperValue in pairs(v) do
					if iTFConfig[k][deeperKey] == nil then
						iTFConfig[k][deeperKey] = deeperValue
					elseif type(deeperValue) == 'table' then
						for inceptionLevelKey, inceptionLevelValue in pairs(deeperValue) do
							if iTFConfig[k][deeperKey][inceptionLevelKey] == nil then
								iTFConfig[k][deeperKey][inceptionLevelKey] = inceptionLevelValue
							elseif type(inceptionLevelValue) == 'table' then
								for wtfKey, wtfValue in pairs(inceptionLevelValue) do
									if iTFConfig[k][deeperKey][inceptionLevelKey][wtfKey] == nil then
										iTFConfig[k][deeperKey][inceptionLevelKey][wtfKey] = wtfValue
									end
								end
							end
						end
					end
				end
			end
		end
	end
end
local function loadProfile(profile)
	if profile == 'default' then
		iTF:LoadDefaults(true)
	end
end

local function MainFrameBG(show)
	if show then
		iTF.mainFrame.tex:Show()
		iTF.show = true
	else
		iTF.mainFrame.tex:Hide()
		iTF.show = nil
	end
end

local options = {
	type = "group",
	args = {
		ShowBackground = {
			name = 'Show background',
			desc = 'Show max size',
			type = 'toggle',
			set = function(info,val) MainFrameBG(val) end,
			get = function(info) return iTF.show end
		},
		--[[
		toggleLock = {
			name = 'Toggle Lock',
			desc = 'lock frame',
			type = 'toggle',
			set = function(info,val) MainFrameBG(val) end,
			get = function(info) return iTF.show end
		},
		--]]
		frameOptions={
			name = 'Frame',
			type = 'group',
			order = 1,
			args = {
				xPos = {
					name = 'Horizontal position',
					desc = 'Horizontal position',
					type = 'range',
					order = 1,
					min = 0,
					max = math.floor(GetScreenWidth()),
					step = 1,
					
					set = function(info,val)
						iTFConfig.anchor.x = val
						iTF:updatePos(true)
					end,
					get = function(info) return iTFConfig.anchor.x end
				},
				yPos = {
					name = 'Vertical position',
					desc = 'Vertical position',
					type = 'range',
					order = 2,
					min = 0,
					max = math.floor(GetScreenHeight()),
					step = 1,
					set = function(info,val)
						iTFConfig.anchor.y = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.anchor.y end
				},
				rows = {
					name = 'Rows',
					desc = 'Row size',
					type = 'range',
					order = 3,
					min = 1,
					max = 30,
					step = 1,
					set = function(info,val)
						iTFConfig.colS = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.colS end
				},
				grow = {
					name = 'Grow',
					desc = 'Grow to',
					type = 'select',
					order = 4,
					values = {['TOPRIGHT'] = 'Down & Left', ['TOPLEFT'] = 'Down & Right', ['BOTTOMRIGHT'] = 'Up & Left', ['BOTTOMLEFT'] = 'Up & Right'},
					set = function(info,val)
						iTFConfig.grow = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.grow end
				},
				spacing = {
					name = 'Spacing',
					desc = 'frameSpacing',
					type = 'range',
					order = 5,
					min = 2,
					max = 100,
					step = 1,
					set = function(info,val)
						iTFConfig.frame.spacing = val
						iTF:updatePos()
					end,
					get = function(info) return iTFConfig.frame.spacing end
				},
				size = {
					name = 'Size',
					type = 'group',
					order = 1,
					args = {
						width = {
							name = 'Width',
							desc = 'Unit frame width',
							type = 'range',
							order = 1,
							min = 10,
							max = 200,
							step = 1,
							
							set = function(info,val)
								iTFConfig.frame.width = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.width end
						},
						height = {
							name = 'Height',
							desc = 'Unit frame height',
							type = 'range',
							order = 2,
							min = 10,
							max = 200,
							step = 1,
							
							set = function(info,val)
								iTFConfig.frame.height = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.height end
						},
					},
				},
				text = {
					name = 'Text',
					type = 'group',
					order = 2,
					args = {
						textPosition = {
							name = 'Text Position',
							desc = 'Text Position',
							type = 'select',
							order = 1,
							values = {['TOPLEFT'] = 'Top left', ['TOPRIGHT'] = 'Top right', ['TOP'] = 'Top', ['LEFT'] = 'Left', ['CENTER'] = 'Center', ['RIGHT'] = 'Right', ['BOTTOMRIGHT'] = 'Bottom right', ['BOTTOMLEFT'] = 'Bottom left', ['BOTTOM'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.text.pos = val
								iTF:updateText()
							end,
							get = function(info) return iTFConfig.text.pos end
						},
					},
				},
				castBar = {
					name = 'Cast bar',
					type = 'group',
					order = 3,
					args = {
						height = {
							name = 'Height',
							desc = 'Cast bar height',
							type = 'range',
							order = 1,
							min = 1,
							max = 60,
							step = 1,
							set = function(info,val)
								iTFConfig.frame.castBarHeight = val
								iTF:updateSize()
							end,
							get = function(info) return iTFConfig.frame.castBarHeight end
						},
					},
				},
				texture = {
					name = 'Texture',
					type = 'group',
					order = 2,
					args = {
						
					},
				},
			},
		},
		--[[
		auras={
			name = 'auras',
			type = 'group',
			order = 3,
			args = {
				-- more options go here
			},
		},
		--]]
		glow={
			name = 'Glows',
			type = 'group',
			order = 4,
			args = {
				outOfCombat = {
					name = 'Out of combat',
					type = 'group',
					order = 1,
					args = {
						enable = {
							name = 'Enable',
							desc = 'Enable out of combat glow',
							order = 1,
							type = 'toggle',
							set = function(info,val) iTFConfig.glows.outOfCombat.enable = val end,
							get = function(info) return iTFConfig.glows.outOfCombat.enable end,
						},
						pos = {
							name = 'Position',
							type = 'select',
							order = 2,
							values = {['glowLeft'] = 'Left', ['glowRight'] = 'Right', ['glowTop'] = 'Top', ['glowBottom'] = 'Bottom'},
							set = function(info,val)
								iTFConfig.glows.outOfCombat.pos = val
								iTF:updateFrames('glow')
							end,
							get = function(info) return iTFConfig.glows.outOfCombat.pos end,
						},
						color = {
							name = 'Color',
							type = 'color',
							order = 3,
							hasAlpha = true,
							set = function(info,r,g,b,a)
								iTFConfig.glows.outOfCombat.color = {r,g,b,a}
								iTF:updateFrames('glow')
							end,
							get = function(info)
								return unpack(iTFConfig.glows.outOfCombat.color)
							end,
						},
					},
				},
			},
		},
		layouts={
			name = 'Layouts',
			type = 'group',
			order = 5,
			args = {
				defaults = {
					order = 1,
					name = 'Reset to Defaults',
					type = 'execute',
					func = function() 
						loadProfile('default')
						iTF:updateFrames()
					end,
				},
			},
		},
	},
}
LibStub('AceConfig-3.0'):RegisterOptionsTable('iTargetingFrames', options)
local AceConfigDialog = LibStub('AceConfigDialog-3.0')
AceConfigDialog:SetDefaultSize('iTargetingFrames', 600,500)

function iTF:toggleConfig(hide)
	AceConfigDialog:Open('iTargetingFrames')
end
